package approve_former

type Node struct {
	ID         string        `json:"id"`
	NodeLevel  int           `json:"-"`
	NamePrefix string        `json:"-"`
	Name       string        `json:"name"`
	Type       string        `json:"type"`
	Props      Props         `json:"props"`
	Children   *Node         `json:"children,omitempty"`
	Branchs    []Branch      `json:"branchs,omitempty"`
	ParentID   string        `json:"ParentID,omitempty"`
	Subjects   []interface{} `json:"subjects,omitempty"`
	Error      bool          `json:"error"`
	NodeType   int           `json:"nodeType,omitempty"`
	Priority   int           `json:"priorityLevel,omitempty"`
}

type Props struct {
	AssignedUser     []UserRole    `json:"assignedUser,omitempty"`
	FormPerms        []FormPerm    `json:"formPerms,omitempty"`
	AssignedType     string        `json:"assignedType,omitempty"`
	Mode             string        `json:"mode,omitempty"`
	Sign             bool          `json:"sign,omitempty"`
	Nobody           Nobody        `json:"nobody,omitempty"`
	TimeLimit        TimeLimit     `json:"timeLimit,omitempty"`
	AssignedRole     []UserRole    `json:"assignedRole,omitempty"`
	SelfSelect       SelfSelect    `json:"selfSelect,omitempty"`
	LeaderTop        LeaderTop     `json:"leaderTop,omitempty"`
	Leader           Leader        `json:"leader,omitempty"`
	Role             []interface{} `json:"role,omitempty"`
	Refuse           Refuse        `json:"refuse,omitempty"`
	FormUser         string        `json:"formUser,omitempty"`
	PassHttp         HTTPRequest   `json:"passHttp,omitempty"`
	RejectHttp       HTTPRequest   `json:"rejectHttp,omitempty"`
	ShouldAdd        bool          `json:"shouldAdd,omitempty"`
	CCSelfSelectFlag int           `json:"ccSelfSelectFlag,omitempty"`
}

type UserRole struct {
	Type      string `json:"type"`
	RoleCode  string `json:"roleCode,omitempty"`
	Name      string `json:"name"`
	ID        string `json:"id"`
	SubjectID string `json:"subjectId,omitempty"`
}

type FormPerm struct {
	ID       string `json:"id"`
	Title    string `json:"title"`
	Required bool   `json:"required"`
	Perm     string `json:"perm"`
}

type Nobody struct {
	Handler      string     `json:"handler"`
	AssignedUser []UserRole `json:"assignedUser,omitempty"`
	AssignedRole []UserRole `json:"assignedRole,omitempty"`
}

type TimeLimit struct {
	Timeout struct {
		Unit  string `json:"unit"`
		Value int    `json:"value"`
	} `json:"timeout"`
	Handler struct {
		Type   string `json:"type"`
		Notify struct {
			Once bool `json:"once"`
			Hour int  `json:"hour"`
		} `json:"notify"`
	} `json:"handler"`
}

type SelfSelect struct {
	Multiple bool `json:"multiple"`
}

type LeaderTop struct {
	EndCondition string `json:"endCondition"`
	EndLevel     int    `json:"endLevel"`
}

type Leader struct {
	Level int `json:"level"`
}

type Refuse struct {
	Type   string `json:"type"`
	Target string `json:"target"`
}

type HTTPRequest struct {
	Method          string      `json:"method"`
	URL             string      `json:"url"`
	Headers         []HTTPParam `json:"headers"`
	ContentType     string      `json:"contentType"`
	Params          []HTTPParam `json:"params"`
	Retry           int         `json:"retry"`
	HandlerByScript bool        `json:"handlerByScript"`
	Success         string      `json:"success"`
	Fail            string      `json:"fail"`
}

type HTTPParam struct {
	Name    string `json:"name"`
	IsField bool   `json:"isField"`
	Value   string `json:"value"`
}

type Branch struct {
	ID        string     `json:"id"`
	Name      string     `json:"name"`
	Type      string     `json:"type"`
	Props     Props      `json:"props"`
	Children  *Node      `json:"children"`
	CondGroup *CondGroup `json:"condGroup,omitempty"`
}

type CondGroup struct {
	Type  string     `json:"type"`
	Items []CondItem `json:"items"`
}

type CondItem struct {
	ShowType     int    `json:"showType"`
	ColumnID     string `json:"columnId"`
	Type         int    `json:"type"`
	ShowName     string `json:"showName"`
	OptType      string `json:"optType"`
	Zdy1         string `json:"zdy1"`
	Opt1         string `json:"opt1"`
	Zdy2         string `json:"zdy2"`
	Opt2         string `json:"opt2"`
	ColumnDbName string `json:"columnDbname"`
	ColumnType   string `json:"columnType"`
	FieldID      string `json:"fieldId"`
	Operator     string `json:"operator"`
	RightValue   string `json:"rightValue"`
}
